import React, { FC } from 'react';
import { Field, RadioButtonGroup, useStyles2 } from '@grafana/ui';
import { RadioButtonSize } from '@grafana/ui/src/components/Forms/RadioButtonGroup/RadioButton';
import { ReportOptions, ReportOrientation, ReportLayout, reportOrientations, reportLayouts } from '../types';
import { GrafanaTheme2 } from '@grafana/data';
import { css } from '@emotion/css';

type Options = Pick<ReportOptions, 'orientation' | 'layout'>;
interface Props {
  options: Options;
  onChange: (options: Options) => void;
  size?: RadioButtonSize;
}

const descriptions = new Map<ReportLayout, string>([
  ['grid', 'Display the panels in their positions on the dashboard.'],
  ['simple', 'Display one panel per row.'],
]);

export const ReportOptionsPicker: FC<Props> = ({ onChange, options, size = 'md' }) => {
  const styles = useStyles2(getStyles);
  const orientation = options.orientation || 'landscape';
  const layout = options.layout || 'grid';

  const onOrientationChange = (newOrientation?: ReportOrientation) => {
    const newOptions: Options = {
      ...options,
      orientation: newOrientation!,
    };
    onChange(newOptions);
  };

  const onLayoutChange = (newLayout?: ReportLayout) => {
    const newOptions: Options = {
      ...options,
      layout: newLayout!,
    };
    onChange(newOptions);
  };

  const selectedOrientation = reportOrientations.find((l) => l.value === orientation)?.value;
  const selectedLayout = reportLayouts.find((l) => l.value === layout)?.value;

  return (
    <>
      <Field label="Orientation">
        <RadioButtonGroup
          onChange={onOrientationChange}
          options={reportOrientations}
          value={selectedOrientation}
          size={size}
        />
      </Field>
      <Field label="Layout">
        <RadioButtonGroup onChange={onLayoutChange} options={reportLayouts} value={selectedLayout} size={size} />
      </Field>
      <small className={styles.helpText}>{descriptions.get(selectedLayout || 'simple')}</small>
    </>
  );
};

const getStyles = (theme: GrafanaTheme2) => {
  return {
    helpText: css`
      display: block;
      color: ${theme.colors.text.secondary};
      margin: -${theme.spacing(1)} 0 ${theme.spacing(1)} 0;
    `,
  };
};
